==> 0.0.7.tar.gz: ios/Classes/LoginCompHelper.swift <==
//
//  Function.swift
//  LoginComp
//
//  Created by Grapecity on 7/2/20.
//  Copyright © 2020 Grapecity. All rights reserved.
//

import UIKit
import LocalAuthentication


func getUDID() -> String {
    
    if getDeviceName() == "Simulator" {
        return getDeviceName()
    }
    
    guard let udid = KeychainService.getUDID() else {
        let newUDID = UIDevice.current.identifierForVendor!.uuidString
        KeychainService.saveUDID(newUDID)
        return newUDID
    }
        
    return udid
}

func getDeviceName() -> String {
    return UIDevice.current.modelName
}

func getOsType() -> String {
    return "IOS"
}

func getOsVersion() -> String {
    return UIDevice.current.systemVersion
}

func getAppVersion() -> String {
    return "51"
}

func getWiFiAddress() -> String {
    var address : String?

    // Get list of all interfaces on the local machine:
    var ifaddr : UnsafeMutablePointer<ifaddrs>?
    guard getifaddrs(&ifaddr) == 0 else { return "0.0.0.0" }
    guard let firstAddr = ifaddr else { return "0.0.0.0" }

    // For each interface ...
    for ifptr in sequence(first: firstAddr, next: { $0.pointee.ifa_next }) {
        let interface = ifptr.pointee

        // Check for IPv4 or IPv6 interface:
        let addrFamily = interface.ifa_addr.pointee.sa_family
        if addrFamily == UInt8(AF_INET) || addrFamily == UInt8(AF_INET6) {

            // Check interface name:
            let name = String(cString: interface.ifa_name)
            if  name == "en0" {

                // Convert interface address to a human readable string:
                var hostname = [CChar](repeating: 0, count: Int(NI_MAXHOST))
                getnameinfo(interface.ifa_addr, socklen_t(interface.ifa_addr.pointee.sa_len),
                            &hostname, socklen_t(hostname.count),
                            nil, socklen_t(0), NI_NUMERICHOST)
                address = String(cString: hostname)
            }
        }
    }
    freeifaddrs(ifaddr)
    
    if let addr = address { return addr }

    return "0.0.0.0"
}

public enum LCNavigation: String {
    case KEY_NAVIGATION_CHANGEPIN
    case KEY_NAVIGATION_CHANGEMOBILE
    case KEY_NAVIGATION_PIN
    case KEY_NAVIGATION_LOGIN
    case unknown
}

enum LCDelegateOperation {
    case success
    case fail
    case goto
    case redirect
    case unknown
}

func getSubURL(_ navigation: LCNavigation) -> String {
    switch navigation {
    case .KEY_NAVIGATION_LOGIN:
        return "/bumm/bid/loginsubmini.aspx"
    case .KEY_NAVIGATION_CHANGEPIN:
        return "/bumm/bid/changecustpin.aspx"
    case .KEY_NAVIGATION_CHANGEMOBILE:
        return "/bumm/bid/bidchangemobile.aspx"
    case .KEY_NAVIGATION_PIN:
        return "/bumm/bid/bidp2p.aspx"
    default:
        return ""
    }
}

let persist = UserDefaults.standard
let KEY_BIOMETRIC_STATUS = "KEY_BIOMETRIC_STATUS"
typealias VoidCompletionBlock = () -> Void

struct LoginHeader {
    static let DEVICE_ID = "deviceId";
    static let DEVICE_IP = "deviceIp";
    static let DEVICE_NAME = "deviceName";
    static let DEVICE_MAC = "deviceMac";
    static let OS_TYPE = "osType";
    static let APP_VERSION = "appVersion";
    static let OS_VERSION = "osVersion";
    static let MOBILE = "mobile";
    static let REMEMBER_ME = "rememberme";
    static let BIOMETRIC_TYPE = "biometricType"
}

func isBiometryAvailable() -> Bool {
    if #available(iOS 8, *) {
        let localAuthenticationContext = LAContext()
        
        var authError: NSError?
        
        if localAuthenticationContext.canEvaluatePolicy(.deviceOwnerAuthenticationWithBiometrics, error: &authError) {
            return true
        }
        else {
            
            guard let error = authError else {
                return false
            }
            
            if #available(iOS 11.0, *) {
                switch(error._code) {
                case LAError.biometryLockout.rawValue, LAError.touchIDNotEnrolled.rawValue:
                    return true
                case LAError.passcodeNotSet.rawValue, LAError.touchIDNotAvailable.rawValue:
                    fallthrough
                default:
                    return false
                }
            } else {
                switch(error._code) {
                case LAError.touchIDNotEnrolled.rawValue:
                    return true
                case LAError.passcodeNotSet.rawValue, LAError.touchIDNotAvailable.rawValue:
                    fallthrough
                default:
                    return false
                }
            }
        }
    } else {
        return false
    }
}

enum eBiometricType {
    case none
    case touchID
    case faceID
}

var BiometricType: eBiometricType {
    get {
        let context = LAContext()
        var error: NSError?
        
        guard context.canEvaluatePolicy(.deviceOwnerAuthenticationWithBiometrics, error: &error) else {
            print(error?.localizedDescription ?? "")
            return .none
        }
        
        if #available(iOS 11.0, *) {
            switch context.biometryType {
            case .none:
                return .none
            case .touchID:
                return .touchID
            case .faceID:
                return .faceID
            @unknown default:
                return .none
            }
        } else {
            return  .touchID
        }
    }
}

//#F9FAFA

func hexStringToUIColor (hex:String, alpha: CGFloat) -> UIColor {
    var cString:String = hex.trimmingCharacters(in: .whitespacesAndNewlines).uppercased()
    
    if (cString.hasPrefix("#")) {
        cString.remove(at: cString.startIndex)
    }
    
    if ((cString.count) != 6) {
        return UIColor.gray
    }
    
    var rgbValue:UInt32 = 0
    Scanner(string: cString).scanHexInt32(&rgbValue)
    
    return UIColor(
        red: CGFloat((rgbValue & 0xFF0000) >> 16) / 255.0,
        green: CGFloat((rgbValue & 0x00FF00) >> 8) / 255.0,
        blue: CGFloat(rgbValue & 0x0000FF) / 255.0,
        alpha: CGFloat(alpha)
    )
}

extension UIViewController {
    func showToast(_ text : String, _ isLogin: Bool = false) {
            if text == "" {
                return
            }
            let toastLabel = ToastLabel()
            toastLabel.numberOfLines = 0
            toastLabel.backgroundColor = UIColor.black.withAlphaComponent(0.8)
            toastLabel.textColor = UIColor.white
            toastLabel.textAlignment = .center;
            toastLabel.text = text
            toastLabel.alpha = 1.0
            toastLabel.layer.cornerRadius = 10;
            toastLabel.clipsToBounds  =  true
            toastLabel.layer.zPosition = CGFloat(MAXFLOAT)
            for vw in self.view.subviews {
                if vw.isKind(of: ToastLabel.self) {
                    vw.removeFromSuperview()
                    break
                }
            }
            self.view.addSubview(toastLabel)
            toastLabel.translatesAutoresizingMaskIntoConstraints = false
            toastLabel.centerXAnchor.constraint(equalTo: self.view.centerXAnchor).isActive = true
            toastLabel.leadingAnchor.constraint(greaterThanOrEqualTo: self.view.leadingAnchor, constant: CGFloat(50)).isActive = true
            toastLabel.trailingAnchor.constraint(lessThanOrEqualTo: self.view.trailingAnchor, constant: CGFloat(-50)).isActive = true
            toastLabel.bottomAnchor.constraint(equalTo: self.view.bottomAnchor, constant: CGFloat(-100)).isActive = true
            toastLabel.sizeToFit()
            let length = text.count
        var duration: TimeInterval! = 5.0
            switch length {
            case let x where x <= 50:
                duration = 5.0
            case let x where (x > 50 && x <= 100):
                duration = 7.0
            case let x where x > 100:
                duration = 9.0
            default:
                duration = 5.0
            }
            UIView.animate(withDuration: 1.0, delay: duration, options: .curveEaseOut, animations: {
                toastLabel.alpha = 0.0
            }, completion: {(isCompleted) in
                toastLabel.removeFromSuperview()
            })
        }
}

class ToastLabel: UILabel {
    
    var topInset: CGFloat = CGFloat(5.0)
    var bottomInset: CGFloat = CGFloat(5.0)
    var leftInset: CGFloat = CGFloat(5.0)
    var rightInset: CGFloat = CGFloat(5.0)
    
    var insets: UIEdgeInsets!//.init(top: topInset, left: leftInset, bottom: bottomInset, right: rightInset)
    
    override func drawText(in rect: CGRect) {
        insets = UIEdgeInsets.init(top: topInset, left: leftInset, bottom: bottomInset, right: rightInset)
        super.drawText(in: rect.inset(by: insets))
    }
    
    override var intrinsicContentSize: CGSize {
        let size = super.intrinsicContentSize
        return CGSize(width: size.width + leftInset + rightInset,
                      height: size.height + topInset + bottomInset)
    }
    
    override func textRect(forBounds bounds: CGRect, limitedToNumberOfLines numberOfLines: Int) -> CGRect {
        insets = UIEdgeInsets.init(top: topInset, left: leftInset, bottom: bottomInset, right: rightInset)
        let insetRect = bounds.inset(by: insets)
        let textRect = super.textRect(forBounds: insetRect, limitedToNumberOfLines: numberOfLines)
        let invertedInsets = UIEdgeInsets(top: -insets.top,
                                          left: -insets.left,
                                          bottom: -insets.bottom,
                                          right: -insets.right)
        return textRect.inset(by: invertedInsets)
    }
}

==> 0.0.10.tar.gz: ios/Classes/LoginCompHelper.swift <==
//
//  Function.swift
//  LoginComp
//
//  Created by Grapecity on 7/2/20.
//  Copyright © 2020 Grapecity. All rights reserved.
//

import UIKit
import LocalAuthentication


func getUDID() -> String {
    
    if getDeviceName() == "Simulator" {
        return getDeviceName()
    }
    
    guard let udid = KeychainService.getUDID() else {
        let newUDID = UIDevice.current.identifierForVendor!.uuidString
        KeychainService.saveUDID(newUDID)
        return newUDID
    }
        
    return udid
}

func getDeviceName() -> String {
    return UIDevice.current.modelName
}

func getOsType() -> String {
    return "IOS"
}

func getOsVersion() -> String {
    return UIDevice.current.systemVersion
}

func getAppVersion() -> String {
    return "51"
}

func getWiFiAddress() -> String {
    var address : String?

    // Get list of all interfaces on the local machine:
    var ifaddr : UnsafeMutablePointer<ifaddrs>?
    guard getifaddrs(&ifaddr) == 0 else { return "0.0.0.0" }
    guard let firstAddr = ifaddr else { return "0.0.0.0" }

    // For each interface ...
    for ifptr in sequence(first: firstAddr, next: { $0.pointee.ifa_next }) {
        let interface = ifptr.pointee

        // Check for IPv4 or IPv6 interface:
        let addrFamily = interface.ifa_addr.pointee.sa_family
        if addrFamily == UInt8(AF_INET) || addrFamily == UInt8(AF_INET6) {

            // Check interface name:
            let name = String(cString: interface.ifa_name)
            if  name == "en0" {

                // Convert interface address to a human readable string:
                var hostname = [CChar](repeating: 0, count: Int(NI_MAXHOST))
                getnameinfo(interface.ifa_addr, socklen_t(interface.ifa_addr.pointee.sa_len),
                            &hostname, socklen_t(hostname.count),
                            nil, socklen_t(0), NI_NUMERICHOST)
                address = String(cString: hostname)
            }
        }
    }
    freeifaddrs(ifaddr)
    
    if let addr = address { return addr }

    return "0.0.0.0"
}

public enum LCNavigation: String {
    case KEY_NAVIGATION_CHANGEPIN
    case KEY_NAVIGATION_CHANGEMOBILE
    case KEY_NAVIGATION_PIN
    case KEY_NAVIGATION_LOGIN
    case unknown
}

enum LCDelegateOperation {
    case success
    case fail
    case goto
    case redirect
    case unknown
}

func getSubURL(_ navigation: LCNavigation) -> String {
    switch navigation {
    case .KEY_NAVIGATION_LOGIN:
        return "/login.aspx"
    case .KEY_NAVIGATION_CHANGEPIN:
        return "/changepin.aspx"
    case .KEY_NAVIGATION_CHANGEMOBILE:
        return "/changemobile.aspx"
    case .KEY_NAVIGATION_PIN:
        return "/p2p.aspx"
    default:
        return ""
    }
}

let persist = UserDefaults.standard
let KEY_BIOMETRIC_STATUS = "KEY_BIOMETRIC_STATUS"
typealias VoidCompletionBlock = () -> Void

struct LoginHeader {
    static let DEVICE_ID = "deviceId";
    static let DEVICE_IP = "deviceIp";
    static let DEVICE_NAME = "deviceName";
    static let DEVICE_MAC = "deviceMac";
    static let OS_TYPE = "osType";
    static let APP_VERSION = "appVersion";
    static let OS_VERSION = "osVersion";
    static let MOBILE = "mobile";
    static let REMEMBER_ME = "rememberme";
    static let BIOMETRIC_TYPE = "biometricType"
}

func isBiometryAvailable() -> Bool {
    if #available(iOS 8, *) {
        let localAuthenticationContext = LAContext()
        
        var authError: NSError?
        
        if localAuthenticationContext.canEvaluatePolicy(.deviceOwnerAuthenticationWithBiometrics, error: &authError) {
            return true
        }
        else {
            
            guard let error = authError else {
                return false
            }
            
            if #available(iOS 11.0, *) {
                switch(error._code) {
                case LAError.biometryLockout.rawValue, LAError.touchIDNotEnrolled.rawValue:
                    return true
                case LAError.passcodeNotSet.rawValue, LAError.touchIDNotAvailable.rawValue:
                    fallthrough
                default:
                    return false
                }
            } else {
                switch(error._code) {
                case LAError.touchIDNotEnrolled.rawValue:
                    return true
                case LAError.passcodeNotSet.rawValue, LAError.touchIDNotAvailable.rawValue:
                    fallthrough
                default:
                    return false
                }
            }
        }
    } else {
        return false
    }
}

enum eBiometricType {
    case none
    case touchID
    case faceID
}

var BiometricType: eBiometricType {
    get {
        let context = LAContext()
        var error: NSError?
        
        guard context.canEvaluatePolicy(.deviceOwnerAuthenticationWithBiometrics, error: &error) else {
            print(error?.localizedDescription ?? "")
            return .none
        }
        
        if #available(iOS 11.0, *) {
            switch context.biometryType {
            case .none:
                return .none
            case .touchID:
                return .touchID
            case .faceID:
                return .faceID
            @unknown default:
                return .none
            }
        } else {
            return  .touchID
        }
    }
}

//#F9FAFA

func hexStringToUIColor (hex:String, alpha: CGFloat) -> UIColor {
    var cString:String = hex.trimmingCharacters(in: .whitespacesAndNewlines).uppercased()
    
    if (cString.hasPrefix("#")) {
        cString.remove(at: cString.startIndex)
    }
    
    if ((cString.count) != 6) {
        return UIColor.gray
    }
    
    var rgbValue:UInt32 = 0
    Scanner(string: cString).scanHexInt32(&rgbValue)
    
    return UIColor(
        red: CGFloat((rgbValue & 0xFF0000) >> 16) / 255.0,
        green: CGFloat((rgbValue & 0x00FF00) >> 8) / 255.0,
        blue: CGFloat(rgbValue & 0x0000FF) / 255.0,
        alpha: CGFloat(alpha)
    )
}

extension UIViewController {
    func showToast(_ text : String, _ isLogin: Bool = false) {
            if text == "" {
                return
            }
            let toastLabel = ToastLabel()
            toastLabel.numberOfLines = 0
            toastLabel.backgroundColor = UIColor.black.withAlphaComponent(0.8)
            toastLabel.textColor = UIColor.white
            toastLabel.textAlignment = .center;
            toastLabel.text = text
            toastLabel.alpha = 1.0
            toastLabel.layer.cornerRadius = 10;
            toastLabel.clipsToBounds  =  true
            toastLabel.layer.zPosition = CGFloat(MAXFLOAT)
            for vw in self.view.subviews {
                if vw.isKind(of: ToastLabel.self) {
                    vw.removeFromSuperview()
                    break
                }
            }
            self.view.addSubview(toastLabel)
            toastLabel.translatesAutoresizingMaskIntoConstraints = false
            toastLabel.centerXAnchor.constraint(equalTo: self.view.centerXAnchor).isActive = true
            toastLabel.leadingAnchor.constraint(greaterThanOrEqualTo: self.view.leadingAnchor, constant: CGFloat(50)).isActive = true
            toastLabel.trailingAnchor.constraint(lessThanOrEqualTo: self.view.trailingAnchor, constant: CGFloat(-50)).isActive = true
            toastLabel.bottomAnchor.constraint(equalTo: self.view.bottomAnchor, constant: CGFloat(-100)).isActive = true
            toastLabel.sizeToFit()
            let length = text.count
        var duration: TimeInterval! = 5.0
            switch length {
            case let x where x <= 50:
                duration = 5.0
            case let x where (x > 50 && x <= 100):
                duration = 7.0
            case let x where x > 100:
                duration = 9.0
            default:
                duration = 5.0
            }
            UIView.animate(withDuration: 1.0, delay: duration, options: .curveEaseOut, animations: {
                toastLabel.alpha = 0.0
            }, completion: {(isCompleted) in
                toastLabel.removeFromSuperview()
            })
        }
}

class ToastLabel: UILabel {
    
    var topInset: CGFloat = CGFloat(5.0)
    var bottomInset: CGFloat = CGFloat(5.0)
    var leftInset: CGFloat = CGFloat(5.0)
    var rightInset: CGFloat = CGFloat(5.0)
    
    var insets: UIEdgeInsets!//.init(top: topInset, left: leftInset, bottom: bottomInset, right: rightInset)
    
    override func drawText(in rect: CGRect) {
        insets = UIEdgeInsets.init(top: topInset, left: leftInset, bottom: bottomInset, right: rightInset)
        super.drawText(in: rect.inset(by: insets))
    }
    
    override var intrinsicContentSize: CGSize {
        let size = super.intrinsicContentSize
        return CGSize(width: size.width + leftInset + rightInset,
                      height: size.height + topInset + bottomInset)
    }
    
    override func textRect(forBounds bounds: CGRect, limitedToNumberOfLines numberOfLines: Int) -> CGRect {
        insets = UIEdgeInsets.init(top: topInset, left: leftInset, bottom: bottomInset, right: rightInset)
        let insetRect = bounds.inset(by: insets)
        let textRect = super.textRect(forBounds: insetRect, limitedToNumberOfLines: numberOfLines)
        let invertedInsets = UIEdgeInsets(top: -insets.top,
                                          left: -insets.left,
                                          bottom: -insets.bottom,
                                          right: -insets.right)
        return textRect.inset(by: invertedInsets)
    }
}
